
/*
Tipue Search 4.0
Copyright (c) 2014 Tipue
Tipue Search is released under the MIT License
http://www.tipue.com/search
*/


var tipuesearch_stop_words = ["and", "be", "by", "do", "for", "he", "how", "if", "is", "it", "my", "not", "of", "or", "the", "to", "up", "what", "when", "use", "who", "she", "my", "his", "her"];

var tipuesearch_replace = {"words": [
     {"word": "tipua", "replace_with": "tipue"},
     {"word": "javscript", "replace_with": "javascript"}
]};

var tipuesearch_stem = {"words": [
     {"word": "e-mail", "stem": "email"},
     {"word": "javascript", "stem": "script"},
     {"word": "procedure", "stem": "subroutine"},
     {"word": "procedure", "stem": "function"}
]};

