{% extends "base.html" %}
{% block title %}
All Types &ndash; {{ project }}
{% endblock %}
{% block body %}
      <div class="row">
        <div class="col-lg-12" id='text'>
			 <h1>Derived Types</h1>
			 <table class="table table-striped nostretched">
			 <thead>
			 <tr><th>Type</th><th>Location</th><th>Extends</th><th>Description</th></tr>
			 </thead><tbody>
			 {% for dtype in project.types|sort(attribute='name') %}
			   <tr><td>{{ dtype }}</td><td>{{ dtype.parent }}</td><td>{{ dtype.extends }}</td><td>{{ dtype.meta['summary'] }}</td></tr>
			 {% endfor %}
			 </tbody></table>
             {{ project.typegraph }}
        </div>
      </div>
{% endblock %}
