
/*
Tipue Search 4.0
Copyright (c) 2014 Tipue
Tipue Search is released under the MIT License
http://www.tipue.com/search
*/


/*
#tipue_search_input
{
     font: 13px/1.6 'open sans', sans-serif;
	color: #333;
	padding: 12px 12px 12px 40px;
	width: 170px;
	border: 1px solid #e2e2e2;
	border-radius: 0;
	-moz-appearance: none;
	-webkit-appearance: none;
     box-shadow: none; 
	outline: 0;
	margin: 0;
	background: #fff url('img/search.png') no-repeat 15px 15px;
}
*/

#tipue_search_content
{
     max-width: 650px;
	padding-top: 15px;
	margin: 0;
}
#tipue_search_loading
{
     padding-top: 60px;     
     background: #fff url('img/loader.gif') no-repeat left;
}

#tipue_search_warning_head
{
     font: 300 15px/1.6 'Open Sans', sans-serif;
	color: #555;
}
#tipue_search_warning
{
     font: 300 13px/1.6 'Open Sans', sans-serif;
	color: #333;
	margin: 7px 0;
}
#tipue_search_warning a
{
     color: #36c;
     font-weight: 300;
     text-decoration: none;
}
#tipue_search_warning a:hover
{
     color: #333;  
}
#tipue_search_results_count
{
     font: 300 13px/1.6 'Open Sans', sans-serif;
	color: #333;
}
.tipue_search_content_title
{
     font: 300 25px/1.7 'Open Sans', sans-serif;
     text-rendering: optimizelegibility;
	margin-top: 23px;
}
.tipue_search_content_title a
{
     color: #333;
     text-decoration: none;
}
.tipue_search_content_title a:hover
{
     color: #555;
}
.tipue_search_content_url
{
     font: 300 13px/1.7 'Open Sans', sans-serif;
     word-break: break-all;
     word-break: break-word;
     -webkit-hyphens: auto;
     -moz-hyphens: auto;
     hyphens: auto;
}
.tipue_search_content_url a
{
     color: #06c;
     text-decoration: none;
}
.tipue_search_content_url a:hover
{
     color: #333;   
}
.tipue_search_content_text
{
     font: 300 15px/1.6 'Open Sans', sans-serif;
	color: #555;
     word-break: break-all;
     word-break: break-word;
     -webkit-hyphens: auto;
     -moz-hyphens: auto;
     hyphens: auto;
     margin-top: 3px;
}
.h01
{
     color: #333;
     font-weight: 400;
}

#tipue_search_foot
{
     margin: 51px 0 21px 0;
}
#tipue_search_foot_boxes
{
	padding: 0;
	margin: 0;
     font: 12px/1 'Open Sans', sans-serif;
}
#tipue_search_foot_boxes li
{
	list-style: none;
	margin: 0;
	padding: 0;
	display: inline;
}
#tipue_search_foot_boxes li a
{
	padding: 9px 15px 10px 15px;
     background-color: #f1f1f1;
     border: 1px solid #dcdcdc;
     border-radius: 1px;
	color: #333;
	margin-right: 7px;
	text-decoration: none;
	text-align: center;
}
#tipue_search_foot_boxes li.current
{
	padding: 9px 15px 10px 15px;
     background: #fff;
     border: 1px solid #dcdcdc;
     border-radius: 1px;
	color: #333;
	margin-right: 7px;
	text-align: center;
}
#tipue_search_foot_boxes li a:hover
{
	border: 1px solid #ccc;
	background-color: #f3f3f3;
}
