{% extends "base.html" %}s
{% block title %}{{ src.name }} &ndash; {{ project }}{% endblock title %}
{% block body %}
  {% import 'macros.html' as macros %}
  <div class="row">
    <h1>{{ src.name }}
    <small>Source File</small>
    {% if src.meta['deprecated'] and src.meta['deprecated'].lower() == 'true' %}
	 <span class="label label-danger depwarn">Deprecated</span>
    {% endif %}
    </h1>
	 {{ macros.info_bar(src.name,src.meta,project_url,src.hierarchy) }}    
  </div>
  <div class="row">
    <div class="col-lg-3">
    {{ macros.sidebar(project,src) }}
    </div>
    <div class="col-lg-9" id='text'>
    {{ src.doc }}
    {% if src.doc %}<br>{% endif %}
    <section>
    <h2><span class="anchor" id="src"></span>Source Code</h2>
    {{ src.src }}
    </section>
    </div>
  </div>
{% endblock body %}
