{% extends "base.html" %}
{% block title %}{{ interface.name }} &ndash; {{ project }}{% endblock title %}
{% block body %}
  {% import 'macros.html' as macros %}
  <div class="row">
    <h1>{{ interface.name }}
	 <small>Interface</small>
    {% if interface.meta['deprecated'] and interface.meta['deprecated'].lower() == 'true' %}
    <span class="label label-danger depwarn">Deprecated</span>
    {% endif %}
    </h1>
	 {{ macros.info_bar(interface.name,interface.meta,project_url,interface.hierarchy) }}    
  </div>
  
  <div class="row">
    <div class="col-lg-3">
    {{ macros.sidebar(project,interface) }}
    </div>
    
    <div class="col-lg-9" id='text'>
    <h2>{{ interface.permission }} interface {{ interface.name }}</h2>
	 {{ interface.doc }}
     {% if interface.callsgraph %}
     <h3>Calls</h3>
     {{ interface.callsgraph }}
     {% endif %}
     {% if interface.calledbygraph %}
     <h3>Called By</h3>
     {{ interface.calledbygraph }}
     {% endif %}
    {% if interface.doc or interface.callsgraph or interface.calledbygraph %}<br>{% endif %}
	
	{% if interface.functions|length > 0 %}
	 <h2>Functions</h2>
	 {% for proc in interface.functions %}
    {{ macros.proc_entry(proc) }}
 	 {% endfor %}
  	 {% endif %}

	{% if interface.subroutines|length > 0 %}
	 <h2>Subroutines</h2>
	 {% for proc in interface.subroutines %}
    {{ macros.proc_entry(proc) }}
 	 {% endfor %}
  	 {% endif %}

	{% if interface.modprocs|length > 0 %}
	 <h2>Module Procedures</h2>
	 {% for proc in interface.modprocs %}
    {{ macros.proc_entry(proc.procedure) }}
 	 {% endfor %}
  	 {% endif %}

 </div>
</div>
{% endblock body %}
